-----------------------------------------------------------------------------
--	File	 : /units/xsb2306/xsb2306_script.lua
--
--	Author(s): Resin Smoker, Domino, Berni
--
--	Summary  : Seraphim Thuumshavoh(great destroyer) or "Death Tentacle" point defense  
--
--	Copyright  2024 4DFAF, All rights reserved.
-----------------------------------------------------------------------------

local SStructureUnit = import('/lua/seraphimunits.lua').SStructureUnit
local SDFUltraChromaticBeamGenerator = import('/lua/seraphimweapons.lua').SDFUltraChromaticBeamGenerator02
local EffectTemplate = import('/lua/EffectTemplates.lua')
local WeaponFile = import('/lua/sim/DefaultWeapons.lua')
local DefaultBeamWeapon = WeaponFile.DefaultBeamWeapon
local myDebug = true

XSB2306 = Class(SStructureUnit) {

	Weapons = {
		FingerOfDeath = Class(SDFUltraChromaticBeamGenerator) {
			FxBeamStartPoint = nil,

			OnCreate = function(self)
				SDFUltraChromaticBeamGenerator.OnCreate(self)
				if myDebug then WARN('OnCreate-01') end
			end,

			PlayFxBeamStart = function(self, muzzle)
				if not self.unit:IsDead() then
					if myDebug then	WARN('PlayFxBeam-01') end
					
					--Sync weapons
					if self and not self.unit.MyWep:BeenDestroyed() and self:GetCurrentTarget() then
						if myDebug then WARN('Target Sync via PlayFxBeamStart') end
						self.TargetSync(self.unit.MyWep:GetCurrentTarget())
					end
					
					--Set flag
					self.unit.BeamActive = true
					
					--Charge weapon with energy
					self.unit:SetMaintenanceConsumptionActive()
					
					--Fire
					SDFUltraChromaticBeamGenerator.PlayFxBeamStart(self, muzzle)
					self.unit:AddBeamFX()
					if myDebug then	WARN('PlayFxBeam-02') end
				end
			end,

			PlayFxBeamEnd = function(self, beam)
				if myDebug then WARN('PlayFxBeamEnd-01') end
				
				--Sync weapons
				if self and not self.unit.MyWep:BeenDestroyed() and self:GetCurrentTarget() then
					if myDebug then WARN('Target Sync via PlayFxBeamEnd') end
					self.TargetSync(self.unit.MyWep:GetCurrentTarget())
				end
				
				--Stop drawing power
				self.unit:SetMaintenanceConsumptionInactive()
				
				--Stop firing the beam
				SDFUltraChromaticBeamGenerator.PlayFxBeamEnd(self, beam)
				
				--Kill FX
				self.unit:RemoveBeamFX()
				
				if myDebug then	WARN('PlayFxBeamEnd-02') end
			end,

			PlayFxWeaponUnpackSequence = function(self)	
				if self and not self.unit:BeenDestroyed() and self.unit.Packed then
					if myDebug then	WARN('PlayFxWeaponUnpackSequence-01') end
					self.unit.Packed = false
					
					--Run unpack animation
					DefaultBeamWeapon.PlayFxWeaponUnpackSequence(self)
					
					if self.unit and not self.unit:IsDead() then
						
						--Sync weapons
						if self and not self.unit.MyWep:BeenDestroyed() and self:GetCurrentTarget() then
							if myDebug then WARN('Target Sync via PlayFxWeaponUnpackSequence') end
							self.TargetSync(self.unit.MyWep:GetCurrentTarget())
						end					
											
						--Enable all dummy weapons (i = #2 thru 12), primary (i = #1) is always active
						if not self.unit.DummyWeaponsActive then
							self.unit.DummyWeaponsActive = true
							if myDebug then WARN('Dummy weapons ON via PlayFxWeaponUnpackSequence') end					
							for i = 2, self.unit:GetWeaponCount() do
								local wep = self.unit:GetWeapon(i)
								wep:SetEnabled(true)
							end					
						end
												
						--Increase the collision shape as the unit is now bigger after unpacking
						local bp = self.unit:GetBlueprint()
						self.unit:SetCollisionShape( 'Box', 0, bp.SizeY , 0, bp.SizeX * 0.5 or 1, bp.SizeY * 2 or 1, bp.SizeZ * 0.5 or 1)
						
					end
					if myDebug then WARN('PlayFxWeaponUnpackSequence-02') end						
				end
			end,

			PlayFxWeaponPackSequence = function(self)
				if self and not self.unit:BeenDestroyed() and not self.unit.Packed then
					if myDebug then WARN('PlayFxWeaponPackSequence-01') end	
					
					--Set flag
					self.unit.Packed = true	
									
					--Stop the weapon beam
					if not self.unit:BeenDestroyed() then
						self:ForkThread(self.BeamStop)
					end
					if self.unit and not self.unit:IsDead() then
						
						--Disable all dummy weapons (2-12), primary (1) is always active
						if self.unit.DummyWeaponsActive then
							self.unit.DummyWeaponsActive = false
							if myDebug then WARN('Dummy weapons OFF via PlayFxWeaponPackSequence') end					
							for i = 2, self.unit:GetWeaponCount() do
								local wep = self.unit:GetWeapon(i)
								wep:SetEnabled(false)
							end
						end
						
						--Run repack animation
						DefaultBeamWeapon.PlayFxWeaponPackSequence(self)
						
						--Reset the collision shape as the unit is now smaller after repacking					
						local bp = self.unit:GetBlueprint()
						self.unit:SetCollisionShape( 'Box', 0, bp.SizeY * 0.5, 0, bp.SizeX * 0.5 or 1, bp.SizeY * 0.5 or 1, bp.SizeZ * 0.5 or 1)
	
						if myDebug then WARN('PlayFxWeaponPackSequence-02') end
					end
				end
			end,

			TargetSync = function(self, target)
				if self.unit and not self.unit:IsDead() and target then
					if myDebug then WARN('TargetSync-01') end
					
					--Sync all dummy weapons to main weapon so they track together
					for i = 2, self.unit:GetWeaponCount() do
						local wep = self.unit:GetWeapon(i)
						wep:SetTargetEntity(target)
					end
					
					if myDebug then WARN('TargetSync-02') end
				end
			end,
		}
	},

	OnStopBeingBuilt = function(self,builder,layer)
			SStructureUnit.OnStopBeingBuilt(self,builder,layer)
			--Move all objects surrounding unit
			self.Trash:Add(CreateRotator(self, 'Object01', 'y', nil, 40, 0, 0))
			self.Trash:Add(CreateRotator(self, 'Object02', 'y', nil, 70, 0, 0))
			self.Trash:Add(CreateRotator(self, 'Object03', 'y', nil, -50, 0, 0))
			self.Trash:Add(CreateRotator(self, 'Object04', 'y', nil, -90, 0, 0))
			self.Trash:Add(CreateRotator(self, 'Object05', 'y', nil, -35, 0, 0))
			self.Trash:Add(CreateRotator(self, 'Object06', 'y', nil, -90, 0, 0))
			self.Trash:Add(CreateRotator(self, 'Object07', 'y', nil, 90, 0, 0))
			
			--Global Variables
			self.MyWep = self:GetWeaponByLabel('FingerOfDeath')
			
			--Set flag to prevent undesired conditions
			self.StartUp = true
			
			--Global Boolean for beam bone FX
			self.FxStatus = false
			
			--Global Boolean for beam weapon FX
			self.BeamActive = false
			
			--Weapon Pack / Unpacked status
			self.Packed = true
					
			--Disable all dummy weapons, will re-enable them after the weapon has unpacked. Main weapon is left active.
			for i = 2, self:GetWeaponCount() do
				local wep = self:GetWeapon(i)
				wep:SetEnabled(false)
			end
			
			--Set flag to tract the status of our dummy weapons
			self.DummyWeaponsActive = false
	end,
	
	OnGotTarget = function(self, Weapon)
		SStructureUnit.OnLostTarget(self, Weapon)
		if myDebug then WARN('OnGotTarget') end
	end,	

	AddBeamFX = function(self)
		if myDebug then
			WARN('AddBeamFX-01')
		end
		if self and not self.FxStatus then
			-- Add small Beam FX
			self.FxStatus = true
			self.BeamFxBag = {}
			local army = self:GetArmy()
			for k, v in EffectTemplate.SUltraChromaticBeamGeneratorBeam do
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'base', self, 'seg01', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg01', self, 'seg02', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg02', self, 'seg03', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg03', self, 'seg04', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg04', self, 'seg05', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg05', self, 'seg06', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg06', self, 'seg07', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg07', self, 'seg08', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg08', self, 'seg09', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg09', self, 'seg10', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg10', self, 'seg11', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg11', self, 'seg12', army, v) )
				table.insert(self.BeamFxBag, AttachBeamEntityToEntity(self, 'seg12', self, 'weapon', army, v) )
			end
			-- Clean up
			for k, v in self.BeamFxBag do
				self.Trash:Add(v)
			end
		end
		if myDebug then
			WARN('AddBeamFX-02')
		end
	end,

	RemoveBeamFX = function(self)
		if self and self.FxStatus then
			if myDebug then	WARN('RemoveBeamFX-01') end
			
			-- Remove beam FX
			if self.BeamFxBag then
				for k, v in self.BeamFxBag do
					v:Destroy()
				end
				self.BeamEffectsBag = {}
			end
			self.FxStatus = false
		end
	end,
	
	BeamStop = function(self)
		if myDebug then
			WARN('BeamStop-01')
		end
		
		--Kill the active beam
		if self and self.BeamActive and not self.MyWep:BeenDestroyed() then
			if myDebug then
				WARN('BeamStop-Disabling Beams')
			end
			if self.MyWep.Beams then
				for k, v in self.MyWep.Beams do
					v.Beam:Disable()
				end
			end
					
			--Stop beam audio
			local bp = self.MyWep:GetBlueprint()
			if bp.Audio.BeamStop then
				self.MyWep:PlaySound(bp.Audio.BeamStop)
			end
			if bp.Audio.BeamLoop and self.MyWep.Beams[1].Beam then
				self.MyWep.Beams[1].Beam:SetAmbientSound(nil, nil)
			end
			
			--Purge flag
			self.BeamActive = false								
		end
		
		if myDebug then
			WARN('BeamStop-02')
		end
	end,		

	--Low toss parts for OnKilled death event
	DestructionPartsLowToss = {'weapon','seg01','seg02','seg03','seg04','seg05','seg06','seg07','seg08','seg09','seg10','seg11','seg12'},
	
	--How long after death to destroy our unit
	DeathThreadDestructionWaitTime = 0,

	OnKilled = function(self, instigator, type, overkillRatio)
		if myDebug then	WARN('OnKilled-01') end
		
		--Remove weapon beam and FX		
		if self then
			if not self:BeenDestroyed() then
				self:ForkThread(self.BeamStop)
				self:RemoveBeamFX()

				--Low toss parts of the tenticle but only if its unpacked
				if not self.Packed then
					for k, v in self.DestructionPartsHighToss do --self.DestructionPartsLowToss do
						if self:IsValidBone(v) then
							if myDebug then	WARN('Bone tossed and hidden: ', v)	end
							self:HideBone(v, true)
						end
					end
				end
			end
		end
		SStructureUnit.OnKilled(self, instigator, type, overkillRatio)
		if myDebug then	WARN('OnKilled-02')	end
	end,

}
TypeClass = XSB2306
